C---------------------------------------------------
C                           Spectrum of perturbations: P(k), sigma_8,V(50h-1Mpc)
C     Anatoly Klypin December 1997
C                                                       Holtzman           appoximation  
C                                             or       BBKS-style + Hu&Sugiyama approx
C                            Needs cdm.fit file -- parameters of models and fitting 
C-----------------------------------------------------
C                            Om   = Omega_0                Omc   = Omega_cdm_present
C                            Omb = Omega_baryon     Omnu =Omega_neutrino_present
C                            Ocurv = Omega_curvature_present
C                            hsmall = H_0/100km/s/Mpc = the Hubble constant
C                            ns    = slope of the power spectum (ns=1 for Harr-Zeld)
C                            AEXPN = expansion parameter = 1/(1+z), z =redshift
C                            All calculations are done in real Mpc,
C                            Only final outputs are scaled to  h^{-1} 
C-----------------------------------------------------
      SUBROUTINE GENMOD(step)
      IMPLICIT REAL*8 (A-H,O-Z)
      include 'PMparameters.h'
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      COMMON / Coeff/   deltac22,deltac,rf,Uklow,Ukup
      CHARACTER         Name*16,Answer*3,StartFile*16
      REAL*8              INTG,ns,INPUT,P2,P,Ptophat,Pgauss,Vtophat
      EXTERNAL           INTG,P2,P,Ptophat,Pgauss,Vtophat
      INTEGER            Fitline,step
      DATA                StartFile/'InStart.dat'/
      double precision ro
c      WRITE (*,*) '================< Enter File Name for output file: '
c      READ  (*,'(a)') Name
      OPEN(1,file='startvals.txt')

      OPEN(2,file='cdm.fit') !  parameters of models
      OPEN(UNIT=15,FILE='cosmoparams.txt',status='unknown')


c     Old Model builder part
      READ  (15,*) Sigma8,ns,Omcurv, Fitline,Box,z,ASTEP, NSEED

C                                                     Constants
      PI     =3.1415926535
      AEXPN =1.
      CALL MODEL(Fitline)
      H      =100.*hsmall     ! Hubble constant
      DO a=0.1,1.,0.1
         CALL AGE(t0,GrowthDen,GrowthVel,a)
      ENDDO
      AEXPN =1.
      a =AEXPN
      CALL AGE(t0,GrowthDen_0,GrowthVel_0,a)
      rf    =8./hsmall        ! top-hat for sigma_8
      Sn    = (Sigma8)**2 / 
     &          INTG(Ptophat,1.d-5,2.d+1) ! normalization of P(k)
c                                             bias parameter      
      Sigma8 = sqrt(Sn*(
     &          INTG(Ptophat,1.d-5,2.d+1)))
c      write (*,10) hsmall,Sigma8,ns  ! check if all is self-consistent
      write (1,10) hsmall,Sigma8,ns
 10   Format(' Hubble=',f7.3,' Sigma_8 =',G11.3,
     &       ' Slope n=',f7.2)
      rf =50./hsmall           ! top-hat for bulk velocity
      Veloc = GrowthVel*H*
     &           sqrt(Sn*(INTG(Vtophat,1.d-5,1.d+0)))      
c      write (*,30) Veloc,rf*hsmall
      write (1,30) Veloc,rf*hsmall
 30   Format(' Bulk Velocity =',F8.2,
     .       'km/s for radius top-hat=',f8.1,'h-1Mpc')
      write (1,*) ' k/h        Pk*h^3   Power Spectrum at z=',1/AEXPN-1.
      DO i=1,110
        w =1.e-4*10.**(i/20.)
        Pk0  =P(w)*Sn*hsmall**3*(2.*pi**2) 
        write (1,14) w/hsmall,Pk0
      ENDDO
 14   Format(2G11.4,5G12.4)
C..........................................................Power spectrum and  
C                                                         Amplitude of fluctuations in a Box at redshift z
c      WRITE (*,'(A,$)')
c     &         'Enter Box(h^-1Mpc), Nparticles(1D) and redshift:'
c      READ  (*,*) Box, NROW, z
      ro = Box/(1.d0*NGRID)       ! Scale length for pi simulations
      Box   =Box/hsmall           ! scale it to real Mpc
      Uklow =2.*pi/Box            ! frequency range for integrals
      Ukup  =2.*pi/Box*(NROW/2.)
      AEXPN =1./(1.+z)            ! expansion parameter
      a     =AEXPN
      CALL AGE(t0,GrowthDen,GrowthVel,a)
      sigma =(GrowthDen/GrowthDen_0)*
     &             sqrt(Sn*INTG(P2,Uklow/sqrt(2.),Ukup))
c      WRITE (*,40) z,sigma,NROW,Box*hsmall
 40   format('  z=',f8.3,' delta\rho/rho in box=',f9.5,/
     .       5X,'Particles=',i4,' Box=',f8.2)
      write (1,*) ' k/h        Pk*h^3   Power Spectrum at z=',1/AEXPN-1.
      DO i=1,130
        w =1.e-4*10.**(i/20.)
        Pk0  =(GrowthDen/GrowthDen_0)**2*P(w)*Sn*hsmall**3*(2.*pi**2) 
        write (1,14) w/hsmall,Pk0
      ENDDO
C............................................................................................      
         OPEN(30,file=StartFile)        !  parameters of a run

c         HEADER='N=128x256 L=20h-1CDMsig=0.239 z=30-----------'

         write (30,*) AEXPN
         write (30,*) ASTEP
         write (30,*) sigma
         write (30,*) ns
         write (30,*) Box*hsmall
         write (30,*) Omcurv
         write (30,*) Nseed+step*10
         write (30,*) Fitline

         write (15,*) sigma
         write (15,*) Box*hsmall
         CLOSE (30)
         close(1)
         close(2)
         close(15)

c      ENDIF
      RETURN
      END
c-----------------------------------  Pcold(k)*k^2
      REAL*8 FUNCTION P2(WK)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      COMMON / Coeff/   deltac22,deltac,rf,Uklow,Ukup
      Real*8 ns
        P2=WK**2*P(WK)
      RETURN
      END

C---------------------------------------------
C                                                                 Power spectrum
C                                                                wk = k= in real Mpc
      REAL*8 FUNCTION P(wk)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      COMMON / FACT/    qqscaleb
      Real*8 ns

      Real  k,f_nu,f_baryon,y_d,alpha_nu,omhh,theta_cmb,obhh
      Real  beta_c,sound_horizon,omega,omegal,z_equality
      Real  hubble,z_drag,R_drag,R_equality,k_equality
      Real  p_c, p_cb, f_c, f_nub,q
      Real  TF_master, gamma_eff,P2
		
	hubble = 0.72
		
	f_nu = 0.0
	f_baryon = Omb/Om
	omhh = Om*hubble*hubble
	theta_cmb=2.726/2.7
	obhh = omhh*f_baryon

        z_equality = 2.50e4*omhh*theta_cmb**(-4.) - 1.D0
        k_equality = 0.0746*omhh*theta_cmb**(-2.)

          z_drag = 0.313*omhh**(-0.419)*(1.+0.607*omhh**(0.674))
          z_drag = 1e0 + z_drag*obhh**(0.238*omhh**(0.223))
        z_drag = 1291e0 * omhh**(0.251)/
     &           (1e0 + 0.659*omhh**(0.828)) * z_drag

    	y_d = (1.+z_equality)/(1.+z_drag)

        R_drag = 31.5*obhh*theta_cmb**(-4.)*1000e0/(1e0 + z_drag)
        R_equality = 31.5*obhh*theta_cmb**(-4.)
     &               *1000e0/(1e0 + z_equality)

        sound_horizon = 2./3./k_equality*sqrt(6./R_equality)*
     &      log(( sqrt(1.+R_drag)+sqrt(R_drag+R_equality) )
     &       /(1.+sqrt(R_equality)))

    	 p_c  = -(5.-sqrt(1.+24*(1.-f_nu-f_baryon)))/4.  
	 p_cb = -(5.-sqrt(1.+24*(1.-f_nu)))/4. 
    	 f_c  = 1.-f_nu-f_baryon
   	 f_cb = 1.-f_nu
   	 f_nub= f_nu+f_baryon
		 
	 alpha_nu= (f_c/f_cb)* (2.*(p_c+p_cb)+5.)/(4.*p_cb+5)
	 alpha_nu= alpha_nu*(1.-0.553*f_nub+0.126*f_nub**3)
	 alpha_nu= alpha_nu/(1.-0.193*sqrt(f_nu)+0.169*f_nu)
	 alpha_nu= alpha_nu*(1.+y_d)**(p_c-p_cb)
	 alpha_nu= alpha_nu*(1.+ (p_cb-p_c)/2. *
     *			(1.+1./(4.*p_c+3.)/(4.*p_cb+7.))/(1.+y_d))
	 beta_c=1./(1.-0.949*f_nub)		 
	 
	 q = wk*theta_cmb**2/omhh
	 gamma_eff=(sqrt(alpha_nu) + (1.-sqrt(alpha_nu))/
     *		(1.+(0.43*wk*sound_horizon)**4))

	 q_eff = q/gamma_eff
	 TF_master= dlog(dexp(1.d0)+1.84*beta_c*sqrt(alpha_nu)*q_eff)
	 TF_master = TF_master/(TF_master + q_eff**2*
     *	             (14.4 + 325./(1.+60.5*q_eff**1.11)))

c	No neutrinos for this team	
c	 q_nu = 3.92*q*sqrt(N_nu/f_nu)
c	 TF_master = TF_master*
c     *     (1.+(1.2*f_nu**(0.64)*N_nu**(0.3+0.6*f_nu))/
c     *	   (q_nu**(-1.6)+q_nu**(0.8)))
        sk= sqrt(wk)
        P2 = wk**ns /
     .         (1.+sk*(Par(2)
     .            +sk*(Par(3)
     .            +sk*(Par(4)
     .            +sk*(Par(5) )))) )**(2.*Par(6))	

        P = wk**(ns)*TF_master**2

c        write(*,*) P,P2,P2/P-1,wk
		
c      If(Par(6).ne.0.)Then   ! Holtzman approx
c        sk= sqrt(wk)
c        P = wk**ns /
c     .         (1.+sk*(Par(2)
c     .            +sk*(Par(3)
c     .            +sk*(Par(4)
c     .            +sk*(Par(5) )))) )**(2.*Par(6))
c      Else                   ! BBKS + Sugiyama approx
c        Gammaeff =Om*hsmall/exp(Omb*(1.+sqrt(hsmall/0.5)/Om))
c        Q = wk/hsmall/Gammaeff
c        Q = wk*qqscaleb
c        P = wk**ns*( LOG(1.+Par(1)*Q)/(Par(1)*Q) )**2/
c     .          sqrt(1.+Q*(Par(2)+
c     .                  Q*(Par(3)**2+
c     .                  Q*(Par(4)**3+
c     .                  Q*(Par(5)**4) ))))
c      EndIf
      RETURN
      END
C-------------------------------------------------
C                                  Age of the Universe: t0 (z=0)
C                                  Expansion parameter: a =1/(1+z)
C                                  Growth_Rate_Density at a: GrowthDen
C                                     GrowthDen =\delta\rho/\rho
C                                     normalized to GrowthDen =a for Omega=1
C                                     GrowthDen < 1 at a=1 for Lcdm or OCDM
C                                  Growth_Rate_velocity at a: GrowthVel
C                                     GrowthVel = V_peculiar= (a/delta)(d delta/d a)
      SUBROUTINE AGE(t0,GrowthDen,GrowthVel,a)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (zero =1.d-12)
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      Common /OmegRat/ OmLOm0,OmcOm0
      Real*8  INTG,ns
      External Hnorm,Hage,Hgrow

c      write(*,*) hsmall, Om

      Oml    =Max(Abs(1.-Om-Omcurv),zero)
      OmLOm0 =Oml/Om
      OmcOm0 =Omcurv/Om
      t0     =9.766/hsmall/sqrt(Om)*INTG(Hage,zero,a)
      z      =1./a-1.
      Hubble = hsmall*sqrt(Om/a**3)*Hnorm(a)
         ww  =INTG(Hgrow,zero,a)
      GrowthVel = -(1.5+OmcOm0*a)/Hnorm(a)**2+
     &                 sqrt(a**5)/Hnorm(a)**3/ww
      GrowthDen =2.5*Hnorm(a)/sqrt(a**3)*ww
c         write (*,20) t0,z,a,Hubble,GrowthDen,GrowthVel
         write (1,20) t0,z,a,Hubble,GrowthDen,GrowthVel
 20      format(' Age=',f8.3,' z=',f6.2,' a=',f6.3,
     .          ' Hubble/100=',f8.3,' GrowthRateDen=',f7.4,
     .          ' GrowthRateVelocity=',f7.4)
      Return
      End
c-----------------------------------  P(k)*k^2*gauss(rf)
      REAL*8 FUNCTION Pgauss(WK)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      COMMON / Coeff/   deltac22,deltac,rf,Uklow,Ukup
      Real*8 ns
        Pgauss=WK**2*P(WK)*EXP(-(rf*WK)**2)
      RETURN
      END
c-----------------------------------  P(k)*k^4*gauss(rf)
      REAL*8 FUNCTION P4gauss(WK)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      COMMON / Coeff/   deltac22,deltac,rf,Uklow,Ukup
      Real*8 ns
        P4gauss=WK**4*P(WK)*EXP(-(rf*WK)**2)
      RETURN
      END
C-------------------------------------- P*k^2*Top-Hat Filter
      REAL*8 FUNCTION Ptophat(wk)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      COMMON / Coeff/   deltac22,deltac,rf,Uklow,Ukup
      Real*8 ns
	   IF (wk.lt.1.d-4) THEN
            Ptophat =wk**ns*wk**2
        ELSE
            X      =wk*rf
	         TOPHAT =( (SIN(X)-x*COS(X))*3./X**3 )**2
            Ptophat=P(wk)*wk**2*TOPHAT
        ENDIF
      RETURN
      END
C-------------------------------------- P*Top-Hat Filter
      REAL*8 FUNCTION Vtophat(wk)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      COMMON / Coeff/   deltac22,deltac,rf,Uklow,Ukup
      Real*8 ns
   	IF (wk.lt.1.d-4) THEN
            Vtophat =wk**ns
        ELSE
            X=wk*rf
	    TOPHAT=( (SIN(X)-x*COS(X))*3./X**3 )**2
            Vtophat=P(wk)*TOPHAT
        ENDIF
      RETURN
      END
C-------------------------------------- Linear Pair-wise V
      REAL*8 FUNCTION Vpairwise(wk)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      COMMON / Coeff/   deltac22,deltac,rf,Uklow,Ukup
      Real*8 ns
            X        =wk*rf
            Vpairwise=P(wk)*(1.-sin(X)/X)
      RETURN
      END
C________________________________________Read parameters of the model
C                                 Om       = Omega_0
C                                 Omb     = Omega_baryon
C                                 Omnu  = Omega_neutrino
C                                 hsmall  = hubble constant
C                                 Par      = fitting parameters
C                                 Par(6) = 0  --- bbks-style+Hu&Sugiyama
C                                 Par(6) ne 0 --- holtzman-style
      SUBROUTINE MODEL(Line)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON / TRUNCOM/ Om,Omb,Omc,Omnu,Omcurv,Par(6),Sn,ns,hsmall
      COMMON / FACT/    qqscaleb
      Real*8 ns,INPUT
      Character             Header*79      

c      Line =INPUT(' Enter Line Number in  cdm.fit=   ')
      Read(2,'(A)') Header
      If(Line.gt.2)Then
         Do i=1,Line -2
            Read (2,*) a
         EndDo
      EndIf
      Read (2,*) Om,Omb,Omc,Omnu,hsmall,Par

c      write(*,*) Omcurv, hsmall, " model"
c      Omcurv =INPUT(' Enter Omega curvature at z=0  =>')
      theta = 2.726/2.7  ! = T_cmb/2.7
      Ob0   =Omb/Om      ! Hu&Sugiyama fitting
      Omh2  =Om*hsmall**2
      a1    =(46.9*Omh2)**0.670*(1.+(32.1*Omh2)**(-0.532))
      a2    =(12.0*Omh2)**0.424*(1.+(45.*Omh2)**(-0.582))
      alpha =1./a1**Ob0/a2**(Ob0**3)
      qqscaleb = theta**2/(1.-Ob0)**0.60/sqrt(alpha)/Omh2

c      Write (*,20)Om,Omb,Omc,Omnu,hsmall,Par
      Write (1,20)Om,Omb,Omc,Omnu,hsmall,Par
 20   Format(' Model: Om_0=',F5.3,' Om_baryon=',F5.3,
     .       ' Om_cold=',F5.3,' O_nu=',F5.3,
     .       ' hsmall=',F4.2,/8x,'Parameters=',(6G9.3))
      Return
      End
C-------------------------------------------- Simpson integration
      REAL*8 FUNCTION INTG(FUNC,A,B)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (EPS=5.0d-4, JMAX=22)
      EXTERNAL FUNC
      OST=-1.D30
      OS= -1.D30
      ST =0.
      DO 11 J=1,JMAX
        CALL TRAPZD(FUNC,A,B,ST,J)
        INTG=(4.0d0*ST-OST)/3.0d0
        IF (ABS(INTG-OS).Le.EPS*ABS(OS)) RETURN
        OS=INTG
        OST=ST
11    CONTINUE
      WRITE (*,*)'Integration did not converge'
      RETURN
      END
C----------------------------------------------
      SUBROUTINE TRAPZD(FUNCC,A,B,S,N)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
        SAVE IT
        EXTERNAL FUNCC
      IF (N.EQ.1) THEN
        S=0.5d0*(B-A)*(FUNCC(A)+FUNCC(B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/TNM
        X=A+0.5D0*DEL
        SUM=0.0D0
        DO 11 J=1,IT
          SUM=SUM+FUNCC(X)
          X=X+DEL
11      CONTINUE
        S=0.5D0*(S+(B-A)*SUM/TNM)
        IT=2*IT
      ENDIF
      RETURN
      END

C-------------------------------------------- Simpson integration
      REAL*8 FUNCTION INTG1(FUNC,A,B)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (EPS=1.0d-3, JMAX=22)
      OST=-1.d30
      OS= -1.d30
      ST =0.
      DO 11 J=1,JMAX
        CALL TRAPZD1(FUNC,A,B,ST,J)
        INTG1=(4.0D0*ST-OST)/3.0D0
        IF (ABS(INTG1-OS).Le.EPS*ABS(OS)) RETURN
        OS=INTG1
        OST=ST
11    CONTINUE
c      WRITE (1,*)'Integration did not converge'
      RETURN
      END
C----------------------------------------------
      SUBROUTINE TRAPZD1(FUNCC,A,B,S,N)
C---------------------------------------
	IMPLICIT Real*8 (A-H,O-Z)
        SAVE IT
        EXTERNAL FUNCC
      IF (N.EQ.1) THEN
        S=0.5D0*(B-A)*(FUNCC(A)+FUNCC(B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/TNM
        X=A+0.5D0*DEL
        SUM=0.0D0
        DO 11 J=1,IT
          SUM=SUM+FUNCC(X)
          X=X+DEL
11      CONTINUE
        S=0.5D0*(S+(B-A)*SUM/TNM)
        IT=2*IT
      ENDIF
      RETURN
      END
C---------------------------------- Read in variables      
c$$$      REAL*8 FUNCTION INPUT(text)
c$$$C-----------------------------------------
c$$$      Character text*(*)
c$$$          write (*,'(A,$)')text
c$$$          read (*,*) x
c$$$          INPUT =x
c$$$      Return
c$$$      End
C-----------------------------------------
      REAL*8 FUNCTION sinhh(x)
C-----------------------------------------
      IMPLICIT REAL*8 (A-H,P-Z)
         sinhh =0.5*(exp(MIN(x,33.d+0))-exp(-MIN(x,33.d+0)))
      RETURN
      END
C---------------------------------------
      REAL*8 FUNCTION Hh(x)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
         Hh =(sqrt(x/(1.d+0 +x**3)))**3 
      Return
      End
C---------------------------------------
      REAL*8 FUNCTION Hnorm(x)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      Common /OmegRat/ OmLOm0,OmcOm0
         Hnorm =sqrt(1.d+0 
     &                  +OmLOm0*x**3
     &                  +OmcOm0*x )
      Return
      End
C---------------------------------------
      REAL*8 FUNCTION Hage(x)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
         Hage =sqrt(x)/Hnorm(x) 
      Return
      End
C---------------------------------------
      REAL*8 FUNCTION Hgrow(x)
C---------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
         Hgrow =(sqrt(x)/Hnorm(x))**3 
      Return
      End
